window.Gleam.initSearch([{"doc":"gulid","title":"gulid","content":"# Gleam ULID\n[Universally Unique Lexicographically Sortable Identifier](https://github.com/ulid/spec) implementation in\nGleam.\n\nWhat&#39;s a ULID? Some say it&#39;s a better UUID. In a string form it is shorter (26\ncharacters vs. 32) and sortable.\n\n# Caveats\n1. Only Erlang build target is supported (at the moment)\n\n[![Package Version](https://img.shields.io/hexpm/v/gulid)](https://hex.pm/packages/gulid)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/gulid/)\n\n```sh\ngleam add gulid\n```\n\n# Basic Use\n\n```gleam\nimport gulid.{ new_as_string, new, new_monotonic, from_string_function,\n               to_string_function }\n\npub fn main() {\n  // Quick and dirty ULID string - performance implications\n  let ulid_str = new_as_string()\n  io.println(&quot;ULID is &quot; &lt;&gt; ulid_str)\n\n  // Convert many ULIDs to String with a generator function\n  let to_string = to_string_function()\n  // `to_string_function` returns a function that then can be used to\n  // convert `Ulid` values to `String`. The reason this is done this way is\n  // that Gleam doesn&#39;t have a way to define module scoped global constants\n  // that could use function calls to initialize. Nor does it have module scoped\n  // `let`. Therefore, the only way to have a `private` reused value is to\n  // have it as a capture. So, there is a `let` in `to_string_function`, which\n  // binds an Erlang array with ULID character encodings, captured in returned\n  // function.\n  let bunch_of_ulids = list.map([new(), new(), new(), new(), new()], to_string)\n  io.println(&quot;A bunch of ULIDs:&quot;)\n  io.debug(bunch_of_ulids)\n\n  // Parse ULIDs from string\n  let from_string = from_string_function()\n  // Quick and dirty\n  let assert Ok(ulid) = from_string(&quot;01J9HSAQG7YR6Z16SS7ZTH26WQ&quot;)\n  io.println(&quot;Quick and dirty parsed ULID is&quot;)\n  io.debug(ulid)\n\n  // More proper\n  io.println(&quot;A more properly parsed ULID is&quot;)\n  case from_string(&quot;01J9HS6WA9ZBA045WTNYWAGPM5&quot;) {\n    Ok(ulid) -&gt; {\n      io.debug(ulid)\n      Nil\n    }\n    Error(InvalidLength(error)) | Error(DecodeError(error)) -&gt;\n      io.println(&quot;Oh, noes: &quot; &lt;&gt; error)\n  }\n\n  // Monotonic ULIDs:\n  // 1. Generate initial `Ulid` with `new()`\n  // 2. Then use `new_monotonic(Ulid)` with initial and subsequently generated\n  list.range(0, 9) // We want 10 monotonic ULIDs\n  |&gt; list.scan(new(), fn(ulid, _) { new_monotonic(ulid) })\n  // Convert&#39;em to strings\n  |&gt; list.map(to_string_function())\n  |&gt; io.debug\n}\n```\n# Advanced Use\nIn case one needs to create or outpur ULIDs out of\n- predefined constituents (timestamp and random)\n- binary representation\n\n```gleam\nimport gleam/erlang\nimport gleam/int\nimport gleam/io\nimport gleam/list\nimport gleam/result\nimport gleam/string\nimport gleam/bool\nimport gulid.{from_parts, from_tuple, to_parts, from_bitarray, to_bitarray}\n\npub fn main() {\n  let ulid =\n    from_parts(\n      erlang.system_time(erlang.Millisecond),\n      int.random(99_999_999_999),\n    )\n  io.println(&quot;My ulid made from spare parts:&quot;)\n  io.debug(ulid)\n\n  let #(timestamp, random) = to_parts(ulid)\n  io.println(&quot;Now, extracted its constituent timestamp and random:&quot;)\n  io.println(\n    &quot;\\tTime: &quot;\n    &lt;&gt; { system_time_to_rfc3339(timestamp / 1000) |&gt; from_codepoints },\n  )\n  // io.debug(system_time_to_rfc3339(timestamp / 1000))\n  io.println(&quot;\\tRandom: &quot; &lt;&gt; int.to_string(random))\n\n  let same_ulid = from_tuple(#(timestamp, random))\n\n  io.println(&quot;Same ulids? &quot; &lt;&gt; { bool.to_string(same_ulid == ulid) })\n\n  // Ulid from a binary\n  let assert Ok(bin_ulid) =\n    &lt;&lt;\n      erlang.system_time(erlang.Millisecond):big-48,\n      int.random(9_999_999_999):big-80,\n    &gt;&gt;\n    |&gt; from_bitarray\n  io.println(&quot;Ulid from a bitarray: &quot;)\n  io.debug(bin_ulid)\n\n  // Ulid to a bitarray\n  io.println(&quot;Ulid to binary: &quot;)\n  bin_ulid\n  |&gt; to_bitarray\n  |&gt; io.debug\n}\n\n@external(erlang, &quot;calendar&quot;, &quot;system_time_to_rfc3339&quot;)\nfn system_time_to_rfc3339(seconds_since_epoch: Int) -&gt; List(Int)\n\nfn from_codepoints(code_points: List(Int)) -&gt; String {\n  code_points\n  |&gt; list.map(string.utf_codepoint)\n  |&gt; list.map(fn(res) {\n    let assert Ok(default_codepoint) = string.utf_codepoint(97)\n    result.unwrap(res, default_codepoint)\n  })\n  |&gt; string.from_utf_codepoints\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/gulid&gt;.\n\n## Examples\n\n```sh\ngleam run -m examples/example1   # Run the example one\ngleam run -m examples/example2   # Run the example two\n```\n\n## Development\n\n```sh\ngleam test  # Run the tests\n```\n\n","url":"index.html"},{"doc":"gulid","title":"Ulid","content":"pub opaque type Ulid\n Opaque `Ulid` type.\n\nSynonyms:\ngulid.Ulid\ngulid Ulid","url":"gulid.html#Ulid"},{"doc":"gulid","title":"UlidError","content":"pub type UlidError {\n  DecodeError(mesage: String)\n  InvalidLength(message: String)\n}\n Ulid module errors to be returned in a `Result`\nDecodeError(mesage: String)\n Returned when failed to decode `Ulid` from a `String`\n\nInvalidLength(message: String)\n Returned when input string is of incorrect length\n\nSynonyms:\ngulid.UlidError\ngulid UlidError","url":"gulid.html#UlidError"},{"doc":"gulid","title":"from_bitarray","content":"pub fn from_bitarray(array: BitArray) -&gt; Result(Ulid, UlidError)\n Builds a `Ulid` value from given `BitArray`. Returns `Ok` with `Ulid` value\n on success or `Error(UlidError)` otherwise.\nSynonyms:\ngulid.from_bitarray\ngulid from_bitarray","url":"gulid.html#from_bitarray"},{"doc":"gulid","title":"from_parts","content":"pub fn from_parts(\n  timestamp timestamp: Int,\n  random random: Int,\n) -&gt; Ulid\n Returns a `Ulid` value, build from a given integer timestamp (millis from\n Epoch) and random values\nSynonyms:\ngulid.from_parts\ngulid from_parts","url":"gulid.html#from_parts"},{"doc":"gulid","title":"from_string_function","content":"pub fn from_string_function() -&gt; fn(String) -&gt;\n  Result(Ulid, UlidError)\n Returns a function that decodes a `Ulid` value from a given string. It \n returns `Ok` with the `Ulid` value or `UlidError`.\n\n ## Examples\n\n \n let from_string = from_string_function()\n let ulid_str = &quot;01J9P2J2B0S4T4DFJAJ6RTV1DE&quot;\n let ulid = from_string(ulid_str)\n io.debug(ulid)\n \nSynonyms:\ngulid.from_string_function\ngulid from_string_function","url":"gulid.html#from_string_function"},{"doc":"gulid","title":"from_tuple","content":"pub fn from_tuple(parts: #(Int, Int)) -&gt; Ulid\n Returns a `Ulid` value, built from a given `#(timestamp, random)` tuple.\nSynonyms:\ngulid.from_tuple\ngulid from_tuple","url":"gulid.html#from_tuple"},{"doc":"gulid","title":"new","content":"pub fn new() -&gt; Ulid\n Returns a new `Ulid` created from current system time and strong random.\nSynonyms:\ngulid.new\ngulid new","url":"gulid.html#new"},{"doc":"gulid","title":"new_as_string","content":"pub fn new_as_string() -&gt; String\n Returns a non-monotonic ULID value as a string. Note, this is a shortcut, \n not very good for performance.\nSynonyms:\ngulid.new_as_string\ngulid new_as_string","url":"gulid.html#new_as_string"},{"doc":"gulid","title":"new_monotonic","content":"pub fn new_monotonic(prev_ulid: Ulid) -&gt; Ulid\n Returns a new `Ulid` value based on given previous one according to the \n behavior, described in the ULID spec, but basically, if the previous `Ulid`\n has the same timestamp then increment the least significant bit of its \n random value by 1 (with carry) to produce a new `Ulid` (with the same \n timestamp).\nSynonyms:\ngulid.new_monotonic\ngulid new_monotonic","url":"gulid.html#new_monotonic"},{"doc":"gulid","title":"to_bitarray","content":"pub fn to_bitarray(ulid: Ulid) -&gt; BitArray\n Returns a binary representation of given `Ulid` value\nSynonyms:\ngulid.to_bitarray\ngulid to_bitarray","url":"gulid.html#to_bitarray"},{"doc":"gulid","title":"to_parts","content":"pub fn to_parts(ulid: Ulid) -&gt; #(Int, Int)\n Returns `Ulid` components in a `#(timestamp, random)` tuple.\nSynonyms:\ngulid.to_parts\ngulid to_parts","url":"gulid.html#to_parts"},{"doc":"gulid","title":"to_string_function","content":"pub fn to_string_function() -&gt; fn(Ulid) -&gt; String\n Returns a function than converts a `Ulid` value to string.\n\n ## Eamples\n\n \n let to_string = to_string_function()\n let ulid = new()\n io.debug(to_string(ulid))\n \nSynonyms:\ngulid.to_string_function\ngulid to_string_function","url":"gulid.html#to_string_function"},{"doc":"gulid","title":"gulid","content":" ## ULID Implementation in Gleam\n\n ### Create\n - `new()`: Non-monotonic.\n - `new_monotonic(Ulid)`: Monotonic using previous `Ulid` value\n - Parse ULID from a string:\n ```gleam\n let from_string = from_string_function()\n from_string(new()) |&gt; io.debug\n ```\n - `from_parts(Int, Int)`: Create from a timestamp milliseconds from Epoch\n                           and random.\n\n ### Convert to `String`\n ```gleam\n    let to_string = to_string_function()\n    let ulid = new()\n    io.println(&quot;Ulid: &quot; &lt;&gt; to_string(ulid))\n ```","url":"gulid.html"}]);